@@ -12,46 +12,52 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import inspect
+"""Falcon API class."""
+
 import re
+
 import six
 
-from falcon import api_helpers as helpers
-from falcon import DEFAULT_MEDIA_TYPE
+from falcon import api_helpers as helpers, DEFAULT_MEDIA_TYPE, routing
 from falcon.http_error import HTTPError
 from falcon.http_status import HTTPStatus
 from falcon.request import Request, RequestOptions
-from falcon.response import Response
 import falcon.responders
-from falcon import routing
+from falcon.response import Response, ResponseOptions
 import falcon.status_codes as status
+from falcon.util.misc import get_argnames
 
 
 class API(object):
     """This class is the main entry point into a Falcon-based app.
 
-    Each API instance provides a callable WSGI interface and a routing engine.
+    Each API instance provides a callable WSGI interface and a routing
+    engine.
 
-    Args:
-        media_type (str, optional): Default media type to use as the value for
-            the Content-Type header on responses (default 'application/json').
-        middleware(object or list, optional): One or more objects
-            (instantiated classes) that implement the following middleware
-            component interface::
+    Keyword Arguments:
+        media_type (str): Default media type to use as the
+            value for the Content-Type header on responses (default
+            'application/json'). The ``falcon`` module provides a
+            number of constants for common media types, such as
+            ``falcon.MEDIA_MSGPACK``, ``falcon.MEDIA_YAML``,
+            ``falcon.MEDIA_XML``, etc.
+        middleware(object or list): Either a single object or a list
+            of objects (instantiated classes) that implement the
+            following middleware component interface::
 
                 class ExampleComponent(object):
                     def process_request(self, req, resp):
-                        \"""Process the request before routing it.
+                        \"\"\"Process the request before routing it.
 
                         Args:
                             req: Request object that will eventually be
                                 routed to an on_* responder method.
                             resp: Response object that will be routed to
                                 the on_* responder.
-                        \"""
+                        \"\"\"
 
                     def process_resource(self, req, resp, resource, params):
-                        \"""Process the request and resource *after* routing.
+                        \"\"\"Process the request and resource *after* routing.
 
                         Note:
                             This method is only called when the request matches
@@ -70,10 +76,10 @@ class API(object):
                                 template fields, that will be passed to the
                                 resource's responder method as keyword
                                 arguments.
-                        \"""
+                        \"\"\"
 
-                    def process_response(self, req, resp, resource)
-                        \"""Post-processing of the response (after routing).
+                    def process_response(self, req, resp, resource, req_succeeded)
+                        \"\"\"Post-processing of the response (after routing).
 
                         Args:
                             req: Request object.
@@ -81,27 +87,43 @@ class API(object):
                             resource: Resource object to which the request was
                                 routed. May be None if no route was found
                                 for the request.
-                        \"""
+                            req_succeeded: True if no exceptions were raised
+                                while the framework processed and routed the
+                                request; otherwise False.
+                        \"\"\"
 
-            See also :ref:`Middleware <middleware>`.
+            (See also: :ref:`Middleware <middleware>`)
 
-        request_type (Request, optional): ``Request``-like class to use instead
+        request_type (Request): ``Request``-like class to use instead
             of Falcon's default class. Among other things, this feature
             affords inheriting from ``falcon.request.Request`` in order
             to override the ``context_type`` class variable.
             (default ``falcon.request.Request``)
 
-        response_type (Response, optional): ``Response``-like class to use
+        response_type (Response): ``Response``-like class to use
             instead of Falcon's default class. (default
             ``falcon.response.Response``)
 
-        router (object, optional): An instance of a custom router
+        router (object): An instance of a custom router
             to use in lieu of the default engine.
-            See also: :ref:`Routing <routing>`.
+            (See also: :ref:`Custom Routers <routing_custom>`)
+
+        independent_middleware (bool): Set to ``True`` if response
+            middleware should be executed independently of whether or
+            not request middleware raises an exception (default
+            ``False``).
 
     Attributes:
-        req_options (RequestOptions): A set of behavioral options related to
-            incoming requests.
+        req_options: A set of behavioral options related to incoming
+            requests. (See also: :py:class:`~.RequestOptions`)
+        resp_options: A set of behavioral options related to outgoing
+            responses. (See also: :py:class:`~.ResponseOptions`)
+        router_options: Configuration options for the router. If a
+            custom router is in use, and it does not expose any
+            configurable options, referencing this attribute will raise
+            an instance of ``AttributeError``.
+
+            (See also: :ref:`CompiledRouterOptions <compiled_router_options>`)
     """
 
     # PERF(kgriffs): Reference via self since that is faster than
@@ -117,34 +139,50 @@ class API(object):
 
     __slots__ = ('_request_type', '_response_type',
                  '_error_handlers', '_media_type', '_router', '_sinks',
-                 '_serialize_error', 'req_options', '_middleware')
+                 '_serialize_error', 'req_options', 'resp_options',
+                 '_middleware', '_independent_middleware', '_router_search',
+                 '_static_routes')
 
     def __init__(self, media_type=DEFAULT_MEDIA_TYPE,
                  request_type=Request, response_type=Response,
-                 middleware=None, router=None):
+                 middleware=None, router=None,
+                 independent_middleware=False):
         self._sinks = []
         self._media_type = media_type
+        self._static_routes = []
 
         # set middleware
-        self._middleware = helpers.prepare_middleware(middleware)
+        self._middleware = helpers.prepare_middleware(
+            middleware, independent_middleware=independent_middleware)
+        self._independent_middleware = independent_middleware
 
         self._router = router or routing.DefaultRouter()
+        self._router_search = helpers.make_router_search(self._router)
 
         self._request_type = request_type
         self._response_type = response_type
 
         self._error_handlers = []
         self._serialize_error = helpers.default_serialize_error
-        self.req_options = RequestOptions()
 
-    def __call__(self, env, start_response):
+        self.req_options = RequestOptions()
+        self.resp_options = ResponseOptions()
+
+        self.req_options.default_media_type = media_type
+        self.resp_options.default_media_type = media_type
+
+        # NOTE(kgriffs): Add default error handlers
+        self.add_error_handler(falcon.HTTPError, self._http_error_handler)
+        self.add_error_handler(falcon.HTTPStatus, self._http_status_handler)
+
+    def __call__(self, env, start_response):  # noqa: C901
         """WSGI `app` method.
 
         Makes instances of API callable from a WSGI server. May be used to
         host an API or called directly in order to simulate requests when
         testing the API.
 
-        See also PEP 3333.
+        (See also: PEP 3333)
 
         Args:
             env (dict): A WSGI environment dictionary
@@ -154,78 +192,88 @@ class API(object):
         """
 
         req = self._request_type(env, options=self.req_options)
-        resp = self._response_type()
+        resp = self._response_type(options=self.resp_options)
         resource = None
-        middleware_stack = []  # Keep track of executed components
         params = {}
 
+        dependent_mw_resp_stack = []
+        mw_req_stack, mw_rsrc_stack, mw_resp_stack = self._middleware
+
+        req_succeeded = False
+
         try:
-            # NOTE(kgriffs): Using an inner try..except in order to
-            # address the case when err_handler raises HTTPError.
-
-            # NOTE(kgriffs): Coverage is giving false negatives,
-            # so disabled on relevant lines. All paths are tested
-            # afaict.
             try:
-                # NOTE(ealogar): The execution of request middleware should be
-                # before routing. This will allow request mw to modify path.
-                self._call_req_mw(middleware_stack, req, resp)
-                # NOTE(warsaw): Moved this to inside the try except because it
-                # is possible when using object-based traversal for
-                # _get_responder() to fail.  An example is a case where an
-                # object does not have the requested next-hop child resource.
-                # In that case, the object being asked to dispatch to its
-                # child will raise an HTTP exception signalling the problem,
-                # e.g. a 404.
-                responder, params, resource = self._get_responder(req)
-
-                # NOTE(kgriffs): If the request did not match any route,
-                # a default responder is returned and the resource is
-                # None.
-                if resource is not None:
-                    self._call_rsrc_mw(middleware_stack, req, resp, resource,
-                                       params)
-
-                responder(req, resp, **params)
-                self._call_resp_mw(middleware_stack, req, resp, resource)
-
-            except Exception as ex:
-                for err_type, err_handler in self._error_handlers:
-                    if isinstance(ex, err_type):
-                        err_handler(ex, req, resp, params)
-                        self._call_resp_mw(middleware_stack, req, resp,
-                                           resource)
-
-                        break
-
+                # NOTE(ealogar): The execution of request middleware
+                # should be before routing. This will allow request mw
+                # to modify the path.
+                # NOTE: if flag set to use independent middleware, execute
+                # request middleware independently. Otherwise, only queue
+                # response middleware after request middleware succeeds.
+                if self._independent_middleware:
+                    for process_request in mw_req_stack:
+                        process_request(req, resp)
                 else:
-                    # PERF(kgriffs): This will propagate HTTPError to
-                    # the handler below. It makes handling HTTPError
-                    # less efficient, but that is OK since error cases
-                    # don't need to be as fast as the happy path, and
-                    # indeed, should perhaps be slower to create
-                    # backpressure on clients that are issuing bad
-                    # requests.
+                    for process_request, process_response in mw_req_stack:
+                        if process_request:
+                            process_request(req, resp)
+                        if process_response:
+                            dependent_mw_resp_stack.insert(0, process_response)
 
-                    # NOTE(ealogar): This will executed remaining
-                    # process_response when no error_handler is given
-                    # and for whatever exception. If an HTTPError is raised
-                    # remaining process_response will be executed later.
-                    self._call_resp_mw(middleware_stack, req, resp, resource)
+                # NOTE(warsaw): Moved this to inside the try except
+                # because it is possible when using object-based
+                # traversal for _get_responder() to fail.  An example is
+                # a case where an object does not have the requested
+                # next-hop child resource. In that case, the object
+                # being asked to dispatch to its child will raise an
+                # HTTP exception signalling the problem, e.g. a 404.
+                responder, params, resource, req.uri_template = self._get_responder(req)
+            except Exception as ex:
+                if not self._handle_exception(ex, req, resp, params):
                     raise
+            else:
+                try:
+                    # NOTE(kgriffs): If the request did not match any
+                    # route, a default responder is returned and the
+                    # resource is None. In that case, we skip the
+                    # resource middleware methods.
+                    if resource is not None:
+                        # Call process_resource middleware methods.
+                        for process_resource in mw_rsrc_stack:
+                            process_resource(req, resp, resource, params)
 
-        except HTTPStatus as ex:
-            self._compose_status_response(req, resp, ex)
-            self._call_resp_mw(middleware_stack, req, resp, resource)
+                    responder(req, resp, **params)
+                    req_succeeded = True
+                except Exception as ex:
+                    if not self._handle_exception(ex, req, resp, params):
+                        raise
+        finally:
+            # NOTE(kgriffs): It may not be useful to still execute
+            # response middleware methods in the case of an unhandled
+            # exception, but this is done for the sake of backwards
+            # compatibility, since it was incidentally the behavior in
+            # the 1.0 release before this section of the code was
+            # reworked.
 
-        except HTTPError as ex:
-            self._compose_error_response(req, resp, ex)
-            self._call_resp_mw(middleware_stack, req, resp, resource)
+            # Call process_response middleware methods.
+            for process_response in mw_resp_stack or dependent_mw_resp_stack:
+                try:
+                    process_response(req, resp, resource, req_succeeded)
+                except Exception as ex:
+                    if not self._handle_exception(ex, req, resp, params):
+                        raise
+
+                    req_succeeded = False
 
         #
         # Set status and headers
         #
-        if req.method == 'HEAD' or resp.status in self._BODILESS_STATUS_CODES:
+
+        # NOTE(kgriffs): While not specified in the spec that the status
+        # must be of type str (not unicode on Py27), some WSGI servers
+        # can complain when it is not.
+        resp_status = str(resp.status) if six.PY2 else resp.status
+
+        if req.method == 'HEAD' or resp_status in self._BODILESS_STATUS_CODES:
             body = []
         else:
             body, length = self._get_body(resp, env.get('wsgi.file_wrapper'))
@@ -236,61 +284,42 @@ class API(object):
         # RFC 2616, as commented in that module's source code. The
         # presence of the Content-Length header is not similarly
         # enforced.
-        if resp.status in (status.HTTP_204, status.HTTP_304):
+        if resp_status in (status.HTTP_204, status.HTTP_304):
             media_type = None
         else:
             media_type = self._media_type
 
         headers = resp._wsgi_headers(media_type)
 
-        # Return the response per the WSGI spec
-        start_response(resp.status, headers)
+        # Return the response per the WSGI spec.
+        start_response(resp_status, headers)
         return body
 
+    @property
+    def router_options(self):
+        return self._router.options
+
     def add_route(self, uri_template, resource, *args, **kwargs):
-        """Associates a templatized URI path with a resource.
+        """Associate a templatized URI path with a resource.
 
-        A resource is an instance of a class that defines various
-        "responder" methods, one for each HTTP method the resource
-        allows. Responder names start with `on_` and are named according to
-        which HTTP method they handle, as in `on_get`, `on_post`, `on_put`,
-        etc.
+        Falcon routes incoming requests to resources based on a set of
+        URI templates. If the path requested by the client matches the
+        template for a given route, the request is then passed on to the
+        associated resource for processing.
 
-        If your resource does not support a particular
-        HTTP method, simply omit the corresponding responder and
-        Falcon will reply with "405 Method not allowed" if that
-        method is ever requested.
+        If no route matches the request, control then passes to a
+        default responder that simply raises an instance of
+        :class:`~.HTTPNotFound`.
 
-        Responders must always define at least two arguments to receive
-        request and response objects, respectively. For example::
-
-            def on_post(self, req, resp):
-                pass
-
-        In addition, if the route's template contains field
-        expressions, any responder that desires to receive requests
-        for that route must accept arguments named after the respective
-        field names defined in the template. A field expression consists
-        of a bracketed field name.
-
-        For example, given the following template::
-
-            /user/{name}
-
-        A PUT request to "/user/kgriffs" would be routed to::
-
-            def on_put(self, req, resp, name):
-                pass
-
-        Individual path segments may contain one or more field expressions.
-        For example::
-
-            /repos/{org}/{repo}/compare/{usr0}:{branch0}...{usr1}:{branch1}
+        (See also: :ref:`Routing <routing>`)
 
         Args:
             uri_template (str): A templatized URI. Care must be
                 taken to ensure the template does not mask any sink
-                patterns, if any are registered (see also `add_sink`).
+                patterns, if any are registered.
+
+                (See also: :meth:`~.add_sink`)
+
             resource (instance): Object which represents a REST
                 resource. Falcon will pass "GET" requests to on_get,
                 "PUT" requests to on_put, etc. If any HTTP methods are not
@@ -318,12 +347,60 @@ class API(object):
         if '//' in uri_template:
             raise ValueError("uri_template may not contain '//'")
 
-        method_map = routing.create_http_method_map(resource)
+        method_map = routing.map_http_methods(resource)
+        routing.set_default_responders(method_map)
         self._router.add_route(uri_template, method_map, resource, *args,
                                **kwargs)
 
+    def add_static_route(self, prefix, directory, downloadable=False):
+        """Add a route to a directory of static files.
+
+        Static routes provide a way to serve files directly. This
+        feature provides an alternative to serving files at the web server
+        level when you don't have that option, when authorization is
+        required, or for testing purposes.
+
+        Warning:
+            Serving files directly from the web server,
+            rather than through the Python app, will always be more efficient,
+            and therefore should be preferred in production deployments.
+
+        Static routes are matched in LIFO order. Therefore, if the same
+        prefix is used for two routes, the second one will override the
+        first. This also means that more specific routes should be added
+        *after* less specific ones. For example, the following sequence
+        would result in ``'/foo/bar/thing.js'`` being mapped to the
+        ``'/foo/bar'`` route, and ``'/foo/xyz/thing.js'`` being mapped to the
+        ``'/foo'`` route::
+
+            api.add_static_route('/foo', foo_path)
+            api.add_static_route('/foo/bar', foobar_path)
+
+        Args:
+            prefix (str): The path prefix to match for this route. If the
+                path in the requested URI starts with this string, the remainder
+                of the path will be appended to the source directory to
+                determine the file to serve. This is done in a secure manner
+                to prevent an attacker from requesting a file outside the
+                specified directory.
+
+                Note that static routes are matched in LIFO order, and are only
+                attempted after checking dynamic routes and sinks.
+
+            directory (str): The source directory from which to serve files.
+            downloadable (bool): Set to ``True`` to include a
+                Content-Disposition header in the response. The "filename"
+                directive is simply set to the name of the requested file.
+
+        """
+
+        self._static_routes.insert(
+            0,
+            routing.StaticRoute(prefix, directory, downloadable=downloadable)
+        )
+
     def add_sink(self, sink, prefix=r'/'):
-        """Registers a sink method for the API.
+        """Register a sink method for the API.
 
         If no route matches a request, but the path in the requested URI
         matches a sink prefix, Falcon will pass control to the
@@ -349,8 +426,9 @@ class API(object):
 
                 Warning:
                     If the prefix overlaps a registered route template,
-                    the route will take precedence and mask the sink
-                    (see also `add_route`).
+                    the route will take precedence and mask the sink.
+
+                    (See also: :meth:`~.add_route`)
 
         """
 
@@ -364,7 +442,25 @@ class API(object):
         self._sinks.insert(0, (prefix, sink))
 
     def add_error_handler(self, exception, handler=None):
-        """Registers a handler for a given exception error type.
+        """Register a handler for a given exception error type.
+
+        Error handlers may be registered for any type, including
+        :class:`~.HTTPError`. This feature provides a central location
+        for logging and otherwise handling exceptions raised by
+        responders, hooks, and middleware components.
+
+        A handler can raise an instance of :class:`~.HTTPError` or
+        :class:`~.HTTPStatus` to communicate information about the issue to
+        the client.  Alternatively, a handler may modify `resp`
+        directly.
+
+        Error handlers are matched in LIFO order. In other words, when
+        searching for an error handler to match a raised exception, and
+        more than one handler matches the exception type, the framework
+        will choose the one that was most recently registered.
+        Therefore, more general error handlers (e.g., for the
+        standard ``Exception`` type) should be added first, to avoid
+        masking more specific handlers for subclassed types.
 
         Args:
             exception (type): Whenever an error occurs when handling a request
@@ -387,10 +483,6 @@ class API(object):
                             # Convert to an instance of falcon.HTTPError
                             raise falcon.HTTPError(falcon.HTTP_792)
 
-                Note:
-                    A handler can either raise an instance of ``HTTPError``
-                    or modify `resp` manually in order to communicate
-                    information about the issue to the client.
 
         """
 
@@ -408,16 +500,22 @@ class API(object):
         self._error_handlers.insert(0, (exception, handler))
 
     def set_error_serializer(self, serializer):
-        """Override the default serializer for instances of HTTPError.
+        """Override the default serializer for instances of :class:`~.HTTPError`.
 
-        When a responder raises an instance of HTTPError, Falcon converts
-        it to an HTTP response automatically. The default serializer
-        supports JSON and XML, but may be overridden by this method to
-        use a custom serializer in order to support other media types.
+        When a responder raises an instance of :class:`~.HTTPError`,
+        Falcon converts it to an HTTP response automatically. The
+        default serializer supports JSON and XML, but may be overridden
+        by this method to use a custom serializer in order to support
+        other media types.
 
-        The ``falcon.HTTPError`` class contains helper methods, such as
-        `to_json()` and `to_dict()`, that can be used from within
-        custom serializers. For example::
+        Note:
+            If a custom media type is used and the type includes a
+            "+json" or "+xml" suffix, the default serializer will
+            convert the error to JSON or XML, respectively.
+
+        The :class:`~.HTTPError` class contains helper methods,
+        such as `to_json()` and `to_dict()`, that can be used from
+        within custom serializers. For example::
 
             def my_serializer(req, resp, exception):
                 representation = None
@@ -434,12 +532,7 @@ class API(object):
                     resp.body = representation
                     resp.content_type = preferred
 
-        Note:
-            If a custom media type is used and the type includes a
-            "+json" or "+xml" suffix, the default serializer will
-            convert the error to JSON or XML, respectively. If this
-            is not desirable, a custom error serializer may be used
-            to override this behavior.
+                resp.append_header('Vary', 'Accept')
 
         Args:
             serializer (callable): A function taking the form
@@ -450,8 +543,9 @@ class API(object):
 
         """
 
-        if len(inspect.getargspec(serializer).args) == 2:
+        if len(get_argnames(serializer)) == 2:
             serializer = helpers.wrap_old_error_serializer(serializer)
+
         self._serialize_error = serializer
 
     # ------------------------------------------------------------------------
@@ -459,13 +553,13 @@ class API(object):
     # ------------------------------------------------------------------------
 
     def _get_responder(self, req):
-        """Searches routes for a matching responder.
+        """Search routes for a matching responder.
 
         Args:
             req: The request object.
 
         Returns:
-            A 3-member tuple consisting of a responder callable,
+            tuple: A 3-member tuple consisting of a responder callable,
             a ``dict`` containing parsed path fields (if any were specified in
             the matching route's URI template), and a reference to the
             responder's resource instance.
@@ -483,11 +577,18 @@ class API(object):
 
         path = req.path
         method = req.method
+        uri_template = None
 
-        route = self._router.find(path)
+        route = self._router_search(path, req=req)
 
         if route is not None:
-            resource, method_map, params = route
+            try:
+                resource, method_map, params, uri_template = route
+            except ValueError:
+                # NOTE(kgriffs): Older routers may not return the
+                # template. But for performance reasons they should at
+                # least return None if they don't support it.
+                resource, method_map, params = route
         else:
             # NOTE(kgriffs): Older routers may indicate that no route
             # was found by returning (None, None, None). Therefore, we
@@ -511,12 +612,18 @@ class API(object):
 
                     break
             else:
-                responder = falcon.responders.path_not_found
 
-        return (responder, params, resource)
+                for sr in self._static_routes:
+                    if sr.match(path):
+                        responder = sr
+                        break
+                else:
+                    responder = falcon.responders.path_not_found
+
+        return (responder, params, resource, uri_template)
 
     def _compose_status_response(self, req, resp, http_status):
-        """Composes a response for the given HTTPStatus instance."""
+        """Compose a response for the given HTTPStatus instance."""
 
         # PERF(kgriffs): The code to set the status and headers is identical
         # to that used in _compose_error_response(), but refactoring in the
@@ -531,7 +638,7 @@ class API(object):
         resp.body = http_status.body
 
     def _compose_error_response(self, req, resp, error):
-        """Composes a response for the given HTTPError instance."""
+        """Compose a response for the given HTTPError instance."""
 
         resp.status = error.status
 
@@ -541,38 +648,51 @@ class API(object):
         if error.has_representation:
             self._serialize_error(req, resp, error)
 
-    def _call_req_mw(self, stack, req, resp):
-        """Run process_request middleware methods."""
+    def _http_status_handler(self, status, req, resp, params):
+        self._compose_status_response(req, resp, status)
 
-        for component in self._middleware:
-            process_request, _, _ = component
-            if process_request is not None:
-                process_request(req, resp)
+    def _http_error_handler(self, error, req, resp, params):
+        self._compose_error_response(req, resp, error)
 
-            # Put executed component on the stack
-            stack.append(component)  # keep track from outside
+    def _handle_exception(self, ex, req, resp, params):
+        """Handle an exception raised from mw or a responder.
 
-    def _call_rsrc_mw(self, stack, req, resp, resource, params):
-        """Run process_resource middleware methods."""
+        Args:
+            ex: Exception to handle
+            req: Current request object to pass to the handler
+                registered for the given exception type
+            resp: Current response object to pass to the handler
+                registered for the given exception type
+            params: Responder params to pass to the handler
+                registered for the given exception type
 
-        for component in self._middleware:
-            _, process_resource, _ = component
-            if process_resource is not None:
-                process_resource(req, resp, resource, params)
+        Returns:
+            bool: ``True`` if a handler was found and called for the
+            exception, ``False`` otherwise.
+        """
 
-    def _call_resp_mw(self, stack, req, resp, resource):
-        """Run process_response middleware."""
+        for err_type, err_handler in self._error_handlers:
+            if isinstance(ex, err_type):
+                try:
+                    err_handler(ex, req, resp, params)
+                except HTTPStatus as status:
+                    self._compose_status_response(req, resp, status)
+                except HTTPError as error:
+                    self._compose_error_response(req, resp, error)
 
-        while stack:
-            _, _, process_response = stack.pop()
-            if process_response is not None:
-                process_response(req, resp, resource)
+                return True
+
+        # NOTE(kgriffs): No error handlers are defined for ex
+        # and it is not one of (HTTPStatus, HTTPError), since it
+        # would have matched one of the corresponding default
+        # handlers.
+        return False
 
     # PERF(kgriffs): Moved from api_helpers since it is slightly faster
     # to call using self, and this function is called for most
     # requests.
     def _get_body(self, resp, wsgi_file_wrapper=None):
-        """Converts resp content into an iterable as required by PEP 333
+        """Convert resp content into an iterable as required by PEP 333
 
         Args:
             resp: Instance of falcon.Response
@@ -581,7 +701,7 @@ class API(object):
                 when resp.stream is a file-like object (default None).
 
         Returns:
-            A two-member tuple of the form (iterable, content_length).
+            tuple: A two-member tuple of the form (iterable, content_length).
             The length is returned as ``None`` when unknown. The
             iterable is determined as follows:
 
@@ -620,10 +740,7 @@ class API(object):
                     iterable = wsgi_file_wrapper(stream,
                                                  self._STREAM_BLOCK_SIZE)
                 else:
-                    iterable = iter(
-                        lambda: stream.read(self._STREAM_BLOCK_SIZE),
-                        b''
-                    )
+                    iterable = helpers.CloseableStreamIterator(stream, self._STREAM_BLOCK_SIZE)
             else:
                 iterable = stream
 
