@@ -18,11 +18,14 @@
 from __future__ import print_function
 
 import argparse
-from collections import defaultdict
+from collections import defaultdict, deque
 from decimal import Decimal
 import gc
+import inspect
+import platform
 import random
 import sys
+import tempfile
 import timeit
 
 try:
@@ -42,13 +45,58 @@ try:
 except ImportError:
     pprofile = None
 
+try:
+    import vmprof
+    from vmshare.service import Service
+except ImportError:
+    vmprof = None
+
 from falcon.bench import create  # NOQA
 import falcon.testing as helpers
 
 
-def bench(name, iterations, env, stat_memory):
-    func = create_bench(name, env)
+# NOTE(kgriffs): Based on testing, these values provide a ceiling that's
+# several times higher than fast x86 hardware can achieve today.
+ITER_DETECTION_MAX_ATTEMPTS = 27
+ITER_DETECTION_MULTIPLIER = 1.7
+ITER_DETECTION_STARTING = 3000
 
+# NOTE(kgriffs): Benchmark duration range, in seconds, to target
+ITER_DETECTION_DURATION_MIN = 1.0
+ITER_DETECTION_DURATION_MAX = 5.0
+
+JIT_WARMING_MULTIPLIER = 30
+
+PYPY = platform.python_implementation() == 'PyPy'
+
+BODY = helpers.rand_string(10240, 10240).encode('utf-8') # NOQA
+HEADERS = {'X-Test': 'Funky Chicken'}  # NOQA
+
+
+class StartResponseMockLite(object):
+    """Mock object representing a WSGI `start_response` callable."""
+
+    def __init__(self):
+        self._called = 0
+        self.status = None
+        self.headers = None
+        self.exc_info = None
+
+    def __call__(self, status, headers, exc_info=None):
+        """Implements the PEP-3333 `start_response` protocol."""
+
+        self._called += 1
+
+        self.status = status
+        self.headers = headers
+        self.exc_info = exc_info
+
+    @property
+    def call_count(self):
+        return self._called
+
+
+def bench(func, iterations, stat_memory):
     gc.collect()
     heap_diff = None
 
@@ -62,10 +110,29 @@ def bench(name, iterations, env, stat_memory):
 
     sec_per_req = Decimal(str(total_sec)) / Decimal(str(iterations))
 
-    sys.stdout.write('.')
-    sys.stdout.flush()
+    return (sec_per_req, heap_diff)
 
-    return (name, sec_per_req, heap_diff)
+
+def determine_iterations(func):
+    # NOTE(kgriffs): Algorithm adapted from IPython's magic timeit
+    # function to determine iterations so that 0.2 <= total time < 2.0
+    iterations = ITER_DETECTION_STARTING
+    for __ in range(1, ITER_DETECTION_MAX_ATTEMPTS):
+        gc.collect()
+
+        total_sec = timeit.timeit(
+            func,
+            setup=gc.enable,
+            number=int(iterations)
+        )
+
+        if total_sec >= ITER_DETECTION_DURATION_MIN:
+            assert total_sec < ITER_DETECTION_DURATION_MAX
+            break
+
+        iterations *= ITER_DETECTION_MULTIPLIER
+
+    return int(iterations)
 
 
 def profile(name, env, filename=None, verbose=False):
@@ -83,9 +150,21 @@ def profile(name, env, filename=None, verbose=False):
         print('=' * len(title))
 
     func = create_bench(name, env)
-
     gc.collect()
-    code = 'for x in range(10000): func()'
+
+    num_iterations = 100000
+
+    if PYPY:
+        print('JIT warmup...')
+
+        # TODO(kgriffs): Measure initial time, and keep iterating until
+        # performance increases and then steadies
+        for x in range(num_iterations * JIT_WARMING_MULTIPLIER):
+            func()
+
+        print('Ready.')
+
+    code = 'for x in range({0}): func()'.format(num_iterations)
 
     if verbose:
         if pprofile is None:
@@ -99,22 +178,67 @@ def profile(name, env, filename=None, verbose=False):
                         sort='tottime', filename=filename)
 
 
-BODY = helpers.rand_string(10240, 10240)  # NOQA
-HEADERS = {'X-Test': 'Funky Chicken'}  # NOQA
+def profile_vmprof(name, env):
+    if vmprof is None:
+        print('vmprof not found. Please install vmprof and try again.')
+        return
+
+    func = create_bench(name, env)
+    gc.collect()
+
+    #
+    # Based on: https://github.com/vmprof/vmprof-python/blob/master/vmprof/__main__.py
+    #
+
+    prof_file = tempfile.NamedTemporaryFile(delete=False)
+    filename = prof_file.name
+
+    vmprof.enable(prof_file.fileno())
+
+    try:
+        for __ in range(1000000):
+            func()
+
+    except BaseException as e:
+        if not isinstance(e, (KeyboardInterrupt, SystemExit)):
+            raise
+
+    vmprof.disable()
+
+    service = Service('vmprof.com')
+    service.post({
+        Service.FILE_CPU_PROFILE: filename,
+        Service.FILE_JIT_PROFILE: filename + '.jit',
+        'argv': ' '.join(sys.argv[:]),
+        'VM': platform.python_implementation(),
+    })
+
+    prof_file.close()
+
+
+def exhaust(iterator_or_generator):
+    # from https://docs.python.org/dev/library/itertools.html#itertools-recipes
+    deque(iterator_or_generator, maxlen=0)
 
 
 def create_bench(name, env):
-    srmock = helpers.StartResponseMock()
+    srmock = StartResponseMockLite()
 
     function = name.lower().replace('-', '_')
     app = eval('create.{0}(BODY, HEADERS)'.format(function))
 
     def bench():
         app(env, srmock)
-        if srmock.status != '200 OK':
-            raise AssertionError(srmock.status + ' != 200 OK')
+        assert srmock.status == '200 OK'
 
-    return bench
+    def bench_generator():
+        exhaust(app(env, srmock))
+        assert srmock.status == '200 OK'
+
+    if inspect.isgeneratorfunction(app):
+        return bench_generator
+    else:
+        return bench
 
 
 def consolidate_datasets(datasets):
@@ -146,7 +270,7 @@ def queues_env():
     path = ('/v1/852809/queues/0fd4c8c6-bd72-11e2-8e47-db5ebd4c8125'
             '/claims/db5ebd4c8125')
 
-    qs = 'limit=10&thing=a%20b&x=%23%24'
+    qs = 'limit=10&thing=a+b&x=%23%24'
     return helpers.create_environ(path, query_string=qs,
                                   headers=request_headers)
 
@@ -167,11 +291,32 @@ def run(frameworks, trials, iterations, stat_memory):
 
     print()
 
+    datasets = []
+
     if not frameworks:
         print('Nothing to do.\n')
-        return
+        return datasets
+
+    benchmarks = []
+    for name in frameworks:
+        bm = create_bench(name, get_env(name))
+
+        bm_iterations = iterations if iterations else determine_iterations(bm)
+
+        if PYPY:
+            print('{}: JIT warmup'.format(name))
+
+            # TODO(kgriffs): Measure initial time, and keep iterating until
+            # performance increases and then steadies
+            bench(bm, bm_iterations * JIT_WARMING_MULTIPLIER, False)
+
+        bm_iterations = iterations if iterations else determine_iterations(bm)
+
+        benchmarks.append((name, bm_iterations, bm))
+        print('{}: {} iterations'.format(name, bm_iterations))
+
+    print()
 
-    datasets = []
     for r in range(trials):
         random.shuffle(frameworks)
 
@@ -179,9 +324,18 @@ def run(frameworks, trials, iterations, stat_memory):
                          (r + 1, trials))
         sys.stdout.flush()
 
-        dataset = [bench(framework, iterations,
-                         get_env(framework), stat_memory)
-                   for framework in frameworks]
+        dataset = []
+        for name, bm_iterations, bm in benchmarks:
+            sec_per_req, heap_diff = bench(
+                bm,
+                bm_iterations,
+                stat_memory
+            )
+
+            dataset.append((name, sec_per_req, heap_diff))
+
+            sys.stdout.write('.')
+            sys.stdout.flush()
 
         datasets.append(dataset)
         print('done.')
@@ -192,20 +346,21 @@ def run(frameworks, trials, iterations, stat_memory):
 def main():
     frameworks = [
         'bottle',
+        'django',
         'falcon',
         'falcon-ext',
         'flask',
         'pecan',
-        'werkzeug'
+        'werkzeug',
     ]
 
     parser = argparse.ArgumentParser(description='Falcon benchmark runner')
     parser.add_argument('-b', '--benchmark', type=str, action='append',
                         choices=frameworks, dest='frameworks', nargs='+')
-    parser.add_argument('-i', '--iterations', type=int, default=50000)
-    parser.add_argument('-t', '--trials', type=int, default=3)
+    parser.add_argument('-i', '--iterations', type=int, default=0)
+    parser.add_argument('-t', '--trials', type=int, default=10)
     parser.add_argument('-p', '--profile', type=str,
-                        choices=['standard', 'verbose'])
+                        choices=['standard', 'verbose', 'vmprof'])
     parser.add_argument('-o', '--profile-output', type=str, default=None)
     parser.add_argument('-m', '--stat-memory', action='store_true')
     args = parser.parse_args()
@@ -228,8 +383,12 @@ def main():
 
     # Profile?
     if args.profile:
-        for name in frameworks:
-            profile(name, get_env(name),
+        framework = 'falcon-ext'
+
+        if args.profile == 'vmprof':
+            profile_vmprof(framework, get_env(framework))
+        else:
+            profile(framework, get_env(framework),
                     filename=args.profile_output,
                     verbose=(args.profile == 'verbose'))
 
@@ -240,6 +399,9 @@ def main():
     datasets = run(frameworks, args.trials, args.iterations,
                    args.stat_memory)
 
+    if not datasets:
+        return
+
     dataset = consolidate_datasets(datasets)
     dataset = sorted(dataset, key=lambda r: r[1])
     baseline = dataset[-1][1]
@@ -251,7 +413,7 @@ def main():
         us_per_req = (sec_per_req * Decimal(10 ** 6))
         factor = round_to_int(baseline / sec_per_req)
 
-        print('{3}. {0:.<15s}{1:.>06d} req/sec or {2: >3.2f} μs/req ({4}x)'.
+        print('{3}. {0:.<20s}{1:.>06d} req/sec or {2: >3.2f} μs/req ({4}x)'.
               format(name, req_per_sec, us_per_req, i + 1, factor))
 
     if heapy and args.stat_memory:
@@ -267,5 +429,6 @@ def main():
 
     print()
 
+
 if __name__ == '__main__':
     main()
