@@ -1,16 +1,11 @@
 import random
 
+import pecan
 from pecan import expose, response, request
 
 
-def rand_string(min, max):
-    int_gen = random.randint
-    string_length = int_gen(min, max)
-    return ''.join([chr(int_gen(ord('\t'), ord('~')))
-                    for i in range(string_length)])
-
-
-body = rand_string(10240, 10240)
+_body = pecan.x_test_body
+_headers = pecan.x_test_headers
 
 
 class TestController(object):
@@ -20,10 +15,10 @@ class TestController(object):
     @expose(content_type='text/plain')
     def test(self):
         user_agent = request.headers['User-Agent']  # NOQA
-        limit = request.params['limit']  # NOQA
-        response.headers['X-Test'] = 'Funky Chicken'
+        limit = request.params.get('limit', '10')  # NOQA
+        response.headers.update(_headers)
 
-        return body
+        return _body
 
 
 class HelloController(object):
@@ -36,7 +31,7 @@ class RootController(object):
 
     @expose(content_type='text/plain')
     def index(self):
-        response.headers['X-Test'] = 'Funky Chicken'
-        return body
+        response.headers.update(_headers)
+        return _body
 
     hello = HelloController()
