@@ -12,16 +12,17 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import functools
+"""Hook decorators."""
+
 from functools import wraps
-import inspect
 
 import six
 
-from falcon import HTTP_METHODS
+from falcon import COMBINED_METHODS
+from falcon.util.misc import get_argnames
 
 
-def before(action):
+def before(action, *args, **kwargs):
     """Decorator to execute the given action function *before* the responder.
 
     Args:
@@ -44,13 +45,19 @@ def before(action):
 
                         params['answer'] = 42
 
+        *args: Any additional arguments will be passed to *action* in the
+            order given, immediately following the *req*, *resp*, *resource*,
+            and *params* arguments.
+
+        **kwargs: Any additional keyword arguments will be passed through to
+            *action*.
     """
 
     def _before(responder_or_resource):
         if isinstance(responder_or_resource, six.class_types):
             resource = responder_or_resource
 
-            for method in HTTP_METHODS:
+            for method in COMBINED_METHODS:
                 responder_name = 'on_' + method.lower()
 
                 try:
@@ -67,7 +74,7 @@ def before(action):
                         # variable that is shared between iterations of the
                         # for loop, above.
                         def let(responder=responder):
-                            do_before_all = _wrap_with_before(action, responder)
+                            do_before_all = _wrap_with_before(responder, action, args, kwargs)
 
                             setattr(resource, responder_name, do_before_all)
 
@@ -77,14 +84,14 @@ def before(action):
 
         else:
             responder = responder_or_resource
-            do_before_one = _wrap_with_before(action, responder)
+            do_before_one = _wrap_with_before(responder, action, args, kwargs)
 
             return do_before_one
 
     return _before
 
 
-def after(action):
+def after(action, *args, **kwargs):
     """Decorator to execute the given action function *after* the responder.
 
     Args:
@@ -93,13 +100,19 @@ def after(action):
             reference to the resource class instance associated with the
             request
 
+        *args: Any additional arguments will be passed to *action* in the
+            order given, immediately following the *req*, *resp*, *resource*,
+            and *params* arguments.
+
+        **kwargs: Any additional keyword arguments will be passed through to
+            *action*.
     """
 
     def _after(responder_or_resource):
         if isinstance(responder_or_resource, six.class_types):
             resource = responder_or_resource
 
-            for method in HTTP_METHODS:
+            for method in COMBINED_METHODS:
                 responder_name = 'on_' + method.lower()
 
                 try:
@@ -112,7 +125,7 @@ def after(action):
                     if callable(responder):
 
                         def let(responder=responder):
-                            do_after_all = _wrap_with_after(action, responder)
+                            do_after_all = _wrap_with_after(responder, action, args, kwargs)
 
                             setattr(resource, responder_name, do_after_all)
 
@@ -122,7 +135,7 @@ def after(action):
 
         else:
             responder = responder_or_resource
-            do_after_one = _wrap_with_after(action, responder)
+            do_after_one = _wrap_with_after(responder, action, args, kwargs)
 
             return do_after_one
 
@@ -134,83 +147,106 @@ def after(action):
 # -----------------------------------------------------------------------------
 
 
-def _has_resource_arg(action):
-    """Check if the given action function accepts a resource arg."""
-
-    if isinstance(action, functools.partial):
-        # NOTE(kgriffs): We special-case this, since versions of
-        # Python prior to 3.4 raise an error when trying to get the
-        # spec for a partial.
-        spec = inspect.getargspec(action.func)
-
-    elif inspect.isroutine(action):
-        # NOTE(kgriffs): We have to distinguish between instances of a
-        # callable class vs. a routine, since Python versions prior to
-        # 3.4 raise an error when trying to get the spec from
-        # a callable class instance.
-        spec = inspect.getargspec(action)
-
-    else:
-        spec = inspect.getargspec(action.__call__)
-
-    return 'resource' in spec.args
-
-
-def _wrap_with_after(action, responder):
+def _wrap_with_after(responder, action, action_args, action_kwargs):
     """Execute the given action function after a responder method.
 
     Args:
+        responder: The responder method to wrap.
         action: A function with a signature similar to a resource responder
             method, taking the form ``func(req, resp, resource)``.
-        responder: The responder method to wrap.
+        action_args: Additiona positional agruments to pass to *action*.
+        action_kwargs: Additional keyword arguments to pass to *action*.
     """
 
     # NOTE(swistakm): create shim before checking what will be actually
     # decorated. This helps to avoid excessive nesting
-    if _has_resource_arg(action):
+    if 'resource' in get_argnames(action):
         shim = action
     else:
         # TODO(kgriffs): This decorator does not work on callable
         # classes in Python vesions prior to 3.4.
         #
         # @wraps(action)
-        def shim(req, resp, resource):
-            action(req, resp)
+        def shim(req, resp, resource, *args, **kwargs):
+            action(req, resp, *args, **kwargs)
+
+    responder_argnames = get_argnames(responder)
+    extra_argnames = responder_argnames[2:]  # Skip req, resp
 
     @wraps(responder)
-    def do_after(self, req, resp, **kwargs):
+    def do_after(self, req, resp, *args, **kwargs):
+        if args:
+            _merge_responder_args(args, kwargs, extra_argnames)
+
         responder(self, req, resp, **kwargs)
-        shim(req, resp, self)
+        shim(req, resp, self, *action_args, **action_kwargs)
 
     return do_after
 
 
-def _wrap_with_before(action, responder):
+def _wrap_with_before(responder, action, action_args, action_kwargs):
     """Execute the given action function before a responder method.
 
     Args:
+        responder: The responder method to wrap.
         action: A function with a similar signature to a resource responder
             method, taking the form ``func(req, resp, resource, params)``.
-        responder: The responder method to wrap
+        action_args: Additiona positional agruments to pass to *action*.
+        action_kwargs: Additional keyword arguments to pass to *action*.
     """
 
     # NOTE(swistakm): create shim before checking what will be actually
     # decorated. This allows to avoid excessive nesting
-    if _has_resource_arg(action):
+    if 'resource' in get_argnames(action):
         shim = action
     else:
         # TODO(kgriffs): This decorator does not work on callable
-        # classes in Python vesions prior to 3.4.
+        # classes in Python versions prior to 3.4.
         #
         # @wraps(action)
-        def shim(req, resp, resource, kwargs):
+        def shim(req, resp, resource, params, *args, **kwargs):
             # NOTE(kgriffs): Don't have to pass "self" even if has_self,
             # since method is assumed to be bound.
-            action(req, resp, kwargs)
+            action(req, resp, params, *args, **kwargs)
+
+    responder_argnames = get_argnames(responder)
+    extra_argnames = responder_argnames[2:]  # Skip req, resp
 
     @wraps(responder)
-    def do_before(self, req, resp, **kwargs):
-        shim(req, resp, self, kwargs)
+    def do_before(self, req, resp, *args, **kwargs):
+        if args:
+            _merge_responder_args(args, kwargs, extra_argnames)
+
+        shim(req, resp, self, kwargs, *action_args, **action_kwargs)
         responder(self, req, resp, **kwargs)
 
     return do_before
+
+
+def _merge_responder_args(args, kwargs, argnames):
+    """Merge responder args into kwargs.
+
+    The framework always passes extra args as keyword arguments.
+    However, when the app calls the responder directly, it might use
+    positional arguments instead, so we need to handle that case. This
+    might happen, for example, when overriding a resource and calling
+    a responder via super().
+
+    Args:
+        args (tuple): Extra args passed into the responder
+        kwargs (dict): Keyword args passed into the responder
+        argnames (list): Extra argnames from the responder's
+            signature, ordered as defined
+    """
+
+    # NOTE(kgriffs): Merge positional args into kwargs by matching
+    # them up to the responder's signature. To do that, we must
+    # find out the names of the positional arguments by matching
+    # them in the order of the arguments named in the responder's
+    # signature.
+    for i, argname in enumerate(argnames):
+        # NOTE(kgriffs): extra_argnames may contain keyword arguments,
+        # which wont be in the args list, and are already in the kwargs
+        # dict anyway, so detect and skip them.
+        if argname not in kwargs:
+            kwargs[argname] = args[i]
