@@ -12,9 +12,13 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+"""Utilities for the Request class."""
+
+import io
+
 
 def header_property(wsgi_name):
-    """Creates a read-only header property.
+    """Create a read-only header property.
 
     Args:
         wsgi_name (str): Case-sensitive name of the header as it would
@@ -34,7 +38,7 @@ def header_property(wsgi_name):
     return property(fget)
 
 
-class Body(object):
+class BoundedStream(io.IOBase):
     """Wrap *wsgi.input* streams to make them more robust.
 
     ``socket._fileobject`` and ``io.BufferedReader`` are sometimes used
@@ -72,16 +76,16 @@ class Body(object):
         """Helper function for proxing reads to the underlying stream.
 
         Args:
-            size (int): Maximum number of bytes/characters to read.
-                Will be coerced, if None or -1, to `self.stream_len`. Will
-                likewise be coerced if greater than `self.stream_len`, so
-                that if the stream doesn't follow standard io semantics,
-                the read won't block.
+            size (int): Maximum number of bytes to read. Will be
+                coerced, if None or -1, to the number of remaining bytes
+                in the stream. Will likewise be coerced if greater than
+                the number of remaining bytes, to avoid making a
+                blocking call to the wrapped stream.
             target (callable): Once `size` has been fixed up, this function
                 will be called to actually do the work.
 
         Returns:
-            Data read from the stream, as returned by `target`.
+            bytes: Data read from the stream, as returned by `target`.
 
         """
 
@@ -94,6 +98,18 @@ class Body(object):
         self._bytes_remaining -= size
         return target(size)
 
+    def readable(self):
+        """Always returns ``True``."""
+        return True
+
+    def seekable(self):
+        """Always returns ``False``."""
+        return False
+
+    def writeable(self):
+        """Always returns ``False``."""
+        return False
+
     def read(self, size=None):
         """Read from the stream.
 
@@ -102,7 +118,7 @@ class Body(object):
                 Defaults to reading until EOF.
 
         Returns:
-            Data read from the stream.
+            bytes: Data read from the stream.
 
         """
 
@@ -116,7 +132,7 @@ class Body(object):
                 Defaults to reading until EOF.
 
         Returns:
-            Data read from the stream.
+            bytes: Data read from the stream.
 
         """
 
@@ -130,8 +146,17 @@ class Body(object):
                 Defaults to reading until EOF.
 
         Returns:
-            Data read from the stream.
+            bytes: Data read from the stream.
 
         """
 
         return self._read(hint, self.stream.readlines)
+
+    def write(self, data):
+        """Always raises IOError; writing is not supported."""
+
+        raise IOError('Stream is not writeable')
+
+
+# NOTE(kgriffs): Alias for backwards-compat
+Body = BoundedStream
