@@ -12,20 +12,32 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+"""Response class."""
+
+import mimetypes
+
 from six import PY2
 from six import string_types as STRING_TYPES
 
 # NOTE(tbug): In some cases, http_cookies is not a module
 # but a dict-like structure. This fixes that issue.
 # See issue https://github.com/falconry/falcon/issues/556
-from six.moves import http_cookies
+from six.moves import http_cookies  # NOQA: I202
 
-from falcon.response_helpers import header_property, format_range
-from falcon.response_helpers import is_ascii_encodable
+from falcon import DEFAULT_MEDIA_TYPE
+from falcon.media import Handlers
+from falcon.response_helpers import (
+    format_content_disposition,
+    format_header_value_list,
+    format_range,
+    header_property,
+    is_ascii_encodable,
+)
 from falcon.util import dt_to_http, TimezoneGMT
 from falcon.util.uri import encode as uri_encode
 from falcon.util.uri import encode_value as uri_encode_value
 
+
 SimpleCookie = http_cookies.SimpleCookie
 CookieError = http_cookies.CookieError
 
@@ -38,6 +50,9 @@ class Response(object):
     Note:
         `Response` is not meant to be instantiated directly by responders.
 
+    Keyword Arguments:
+        options (dict): Set of global options passed from the API handler.
+
     Attributes:
         status (str): HTTP status line (e.g., '200 OK'). Falcon requires the
             full status line, not just the code (e.g., 200). This design
@@ -52,10 +67,18 @@ class Response(object):
                 codes. They all start with the ``HTTP_`` prefix, as in:
                 ``falcon.HTTP_204``.
 
-        body (str or unicode): String representing response content. If
-            Unicode, Falcon will encode as UTF-8 in the response. If
-            data is already a byte string, use the data attribute
-            instead (it's faster).
+        media (object): A serializable object supported by the media handlers
+            configured via :class:`falcon.RequestOptions`.
+
+            See :ref:`media` for more information regarding media handling.
+
+        body (str or unicode): String representing response content.
+
+            If set to a Unicode type (``unicode`` in Python 2, or
+            ``str`` in Python 3), Falcon will encode the text as UTF-8
+            in the response. If the content is already a byte string,
+            use the :attr:`data` attribute instead (it's faster).
+
         data (bytes): Byte string representing response content.
 
             Use this attribute in lieu of `body` when your content is
@@ -83,12 +106,29 @@ class Response(object):
             blocks as byte strings. Falcon will use *wsgi.file_wrapper*, if
             provided by the WSGI server, in order to efficiently serve
             file-like objects.
-
         stream_len (int): Expected length of `stream`. If `stream` is set,
             but `stream_len` is not, Falcon will not supply a
             Content-Length header to the WSGI server. Consequently, the
             server may choose to use chunked encoding or one of the
             other strategies suggested by PEP-3333.
+
+        context (dict): Dictionary to hold any data about the response which is
+            specific to your app. Falcon itself will not interact with this
+            attribute after it has been initialized.
+        context_type (class): Class variable that determines the factory or
+            type to use for initializing the `context` attribute. By default,
+            the framework will instantiate standard ``dict`` objects. However,
+            you may override this behavior by creating a custom child class of
+            ``falcon.Response``, and then passing that new class to
+            `falcon.API()` by way of the latter's `response_type` parameter.
+
+            Note:
+                When overriding `context_type` with a factory function (as
+                opposed to a class), the function is called like a method of
+                the current Response instance. Therefore the first argument is
+                the Response instance itself (self).
+
+        options (dict): Set of global options passed from the API handler.
     """
 
     __slots__ = (
@@ -98,22 +138,53 @@ class Response(object):
         '_cookies',
         'status',
         'stream',
-        'stream_len'
+        'stream_len',
+        'context',
+        'options',
+        '__dict__',
     )
 
-    def __init__(self):
+    # Child classes may override this
+    context_type = dict
+
+    def __init__(self, options=None):
         self.status = '200 OK'
         self._headers = {}
 
+        self.options = options if options else ResponseOptions()
+
         # NOTE(tbug): will be set to a SimpleCookie object
         # when cookie is set via set_cookie
         self._cookies = None
+        self._media = None
 
         self.body = None
         self.data = None
         self.stream = None
         self.stream_len = None
 
+        self.context = self.context_type()
+
+    @property
+    def media(self):
+        return self._media
+
+    @media.setter
+    def media(self, obj):
+        self._media = obj
+
+        if not self.content_type:
+            self.content_type = self.options.default_media_type
+
+        handler = self.options.media_handlers.find_by_media_type(
+            self.content_type,
+            self.options.default_media_type
+        )
+        self.data = handler.serialize(self._media)
+
+    def __repr__(self):
+        return '<%s: %s>' % (self.__class__.__name__, self.status)
+
     def set_stream(self, stream, stream_len):
         """Convenience method for setting both `stream` and `stream_len`.
 
@@ -133,7 +204,7 @@ class Response(object):
         self.stream_len = stream_len
 
     def set_cookie(self, name, value, expires=None, max_age=None,
-                   domain=None, path=None, secure=True, http_only=True):
+                   domain=None, path=None, secure=None, http_only=True):
         """Set a response cookie.
 
         Note:
@@ -142,34 +213,74 @@ class Response(object):
 
         See Also:
             To learn more about setting cookies, see
-            :ref:`Setting Cookies <setting-cookies>`. The parameters listed
-            below correspond to those defined in `RFC 6265`_.
+            :ref:`Setting Cookies <setting-cookies>`. The parameters
+            listed below correspond to those defined in `RFC 6265`_.
 
         Args:
-            name (str):
-                Cookie name
-            value (str):
-                Cookie value
-            expires (datetime): Specifies when the cookie should expire. By
-                default, cookies expire when the user agent exits.
-            max_age (int): Defines the lifetime of the cookie in seconds.
-                After the specified number of seconds elapse, the client
-                should discard the cookie. Coercion to `int` is attempted
-                if provided with `float` or `str`.
-            domain (str): Specifies the domain for which the cookie is valid.
-                An explicitly specified domain must always start with a dot.
-                A value of 0 means the cookie should be discarded immediately.
-            path (str): Specifies the subset of URLs to
-                which this cookie applies.
-            secure (bool): Direct the client to only return the cookie in
-                subsequent requests if they are made over HTTPS
-                (default: ``True``). This prevents attackers from reading
-                sensitive cookie data. Note that for the `secure` cookie
-                attribute to be effective, your application will need to
-                enforce HTTPS. See also: `RFC 6265, Section 4.1.2.5`_.
-            http_only (bool): Direct the client to only transfer the cookie
-                with unscripted HTTP requests (default: ``True``). This is
-                intended to mitigate some forms of cross-site scripting.
+            name (str): Cookie name
+            value (str): Cookie value
+
+        Keyword Args:
+            expires (datetime): Specifies when the cookie should expire.
+                By default, cookies expire when the user agent exits.
+
+                (See also: RFC 6265, Section 4.1.2.1)
+            max_age (int): Defines the lifetime of the cookie in
+                seconds. By default, cookies expire when the user agent
+                exits. If both `max_age` and `expires` are set, the
+                latter is ignored by the user agent.
+
+                Note:
+                    Coercion to ``int`` is attempted if provided with
+                    ``float`` or ``str``.
+
+                (See also: RFC 6265, Section 4.1.2.2)
+
+            domain (str): Restricts the cookie to a specific domain and
+                any subdomains of that domain. By default, the user
+                agent will return the cookie only to the origin server.
+                When overriding this default behavior, the specified
+                domain must include the origin server. Otherwise, the
+                user agent will reject the cookie.
+
+                (See also: RFC 6265, Section 4.1.2.3)
+
+            path (str): Scopes the cookie to the given path plus any
+                subdirectories under that path (the "/" character is
+                interpreted as a directory separator). If the cookie
+                does not specify a path, the user agent defaults to the
+                path component of the requested URI.
+
+                Warning:
+                    User agent interfaces do not always isolate
+                    cookies by path, and so this should not be
+                    considered an effective security measure.
+
+                (See also: RFC 6265, Section 4.1.2.4)
+
+            secure (bool): Direct the client to only return the cookie
+                in subsequent requests if they are made over HTTPS
+                (default: ``True``). This prevents attackers from
+                reading sensitive cookie data.
+
+                Note:
+                    The default value for this argument is normally
+                    ``True``, but can be modified by setting
+                    :py:attr:`~.ResponseOptions.secure_cookies_by_default`
+                    via :any:`API.resp_options`.
+
+                Warning:
+                    For the `secure` cookie attribute to be effective,
+                    your application will need to enforce HTTPS.
+
+                (See also: RFC 6265, Section 4.1.2.5)
+
+            http_only (bool): Direct the client to only transfer the
+                cookie with unscripted HTTP requests
+                (default: ``True``). This is intended to mitigate some
+                forms of cross-site scripting.
+
+                (See also: RFC 6265, Section 4.1.2.6)
 
         Raises:
             KeyError: `name` is not a valid cookie name.
@@ -178,9 +289,6 @@ class Response(object):
         .. _RFC 6265:
             http://tools.ietf.org/html/rfc6265
 
-        .. _RFC 6265, Section 4.1.2.5:
-            https://tools.ietf.org/html/rfc6265#section-4.1.2.5
-
         """
 
         if not is_ascii_encodable(name):
@@ -188,9 +296,8 @@ class Response(object):
         if not is_ascii_encodable(value):
             raise ValueError('"value" is not ascii encodable')
 
-        if PY2:
-            name = str(name)
-            value = str(value)
+        name = str(name)
+        value = str(value)
 
         if self._cookies is None:
             self._cookies = SimpleCookie()
@@ -233,8 +340,13 @@ class Response(object):
         if path:
             self._cookies[name]['path'] = path
 
-        if secure:
-            self._cookies[name]['secure'] = secure
+        if secure is None:
+            is_secure = self.options.secure_cookies_by_default
+        else:
+            is_secure = secure
+
+        if is_secure:
+            self._cookies[name]['secure'] = True
 
         if http_only:
             self._cookies[name]['httponly'] = http_only
@@ -242,10 +354,13 @@ class Response(object):
     def unset_cookie(self, name):
         """Unset a cookie in the response
 
-        Note:
-            This will clear the contents of the cookie, and instruct
-            the browser to immediately expire its own copy of the
-            cookie, if any.
+        Clears the contents of the cookie, and instructs the user
+        agent to immediately expire its own copy of the cookie.
+
+        Warning:
+            In order to successfully remove a cookie, both the
+            path and the domain must match the values that were
+            used when the cookie was created.
         """
         if self._cookies is None:
             self._cookies = SimpleCookie()
@@ -284,16 +399,45 @@ class Response(object):
         Args:
             name (str): Header name (case-insensitive). The restrictions
                 noted below for the header's value also apply here.
-            value (str): Value for the header. Must be of type ``str`` or
-                ``StringType`` and contain only ISO-8859-1 characters.
+            value (str): Value for the header. Must be convertable to
+                ``str`` or be of type ``str`` or
+                ``StringType``. Strings must contain only US-ASCII characters.
                 Under Python 2.x, the ``unicode`` type is also accepted,
-                although such strings are also limited to ISO-8859-1.
+                although such strings are also limited to US-ASCII.
         """
-        name, value = self._encode_header(name, value)
+
+        # NOTE(kgriffs): uwsgi fails with a TypeError if any header
+        # is not a str, so do the conversion here. It's actually
+        # faster to not do an isinstance check. str() will encode
+        # to US-ASCII.
+        name = str(name)
+        value = str(value)
 
         # NOTE(kgriffs): normalize name by lowercasing it
         self._headers[name.lower()] = value
 
+    def delete_header(self, name):
+        """Delete a header that was previously set for this response.
+
+        If the header was not previously set, nothing is done (no error is
+        raised).
+
+        Note that calling this method is equivalent to setting the corresponding
+        header property (when said property is available) to ``None``. For
+        example::
+
+            resp.etag = None
+
+        Args:
+            name (str): Header name (case-insensitive).  Must be of type
+                ``str`` or ``StringType`` and contain only US-ASCII characters.
+                Under Python 2.x, the ``unicode`` type is also accepted,
+                although such strings are also limited to US-ASCII.
+        """
+
+        # NOTE(kgriffs): normalize name by lowercasing it
+        self._headers.pop(name.lower(), None)
+
     def append_header(self, name, value):
         """Set or append a header for this response.
 
@@ -308,13 +452,19 @@ class Response(object):
         Args:
             name (str): Header name (case-insensitive). The restrictions
                 noted below for the header's value also apply here.
-            value (str): Value for the header. Must be of type ``str`` or
-                ``StringType`` and contain only ISO-8859-1 characters.
+            value (str): Value for the header. Must be convertable to
+                ``str`` or be of type ``str`` or
+                ``StringType``. Strings must contain only US-ASCII characters.
                 Under Python 2.x, the ``unicode`` type is also accepted,
-                although such strings are also limited to ISO-8859-1.
-
+                although such strings are also limited to US-ASCII.
         """
-        name, value = self._encode_header(name, value)
+
+        # NOTE(kgriffs): uwsgi fails with a TypeError if any header
+        # is not a str, so do the conversion here. It's actually
+        # faster to not do an isinstance check. str() will encode
+        # to US-ASCII.
+        name = str(name)
+        value = str(value)
 
         name = name.lower()
         if name in self._headers:
@@ -332,9 +482,9 @@ class Response(object):
             headers (dict or list): A dictionary of header names and values
                 to set, or a ``list`` of (*name*, *value*) tuples. Both *name*
                 and *value* must be of type ``str`` or ``StringType`` and
-                contain only ISO-8859-1 characters. Under Python 2.x, the
+                contain only US-ASCII characters. Under Python 2.x, the
                 ``unicode`` type is also accepted, although such strings are
-                also limited to ISO-8859-1.
+                also limited to US-ASCII.
 
                 Note:
                     Falcon can process a list of tuples slightly faster
@@ -351,16 +501,22 @@ class Response(object):
         # NOTE(kgriffs): We can't use dict.update because we have to
         # normalize the header names.
         _headers = self._headers
+
         for name, value in headers:
-            name, value = self._encode_header(name, value)
+            # NOTE(kgriffs): uwsgi fails with a TypeError if any header
+            # is not a str, so do the conversion here. It's actually
+            # faster to not do an isinstance check. str() will encode
+            # to US-ASCII.
+            name = str(name)
+            value = str(value)
+
             _headers[name.lower()] = value
 
     def add_link(self, target, rel, title=None, title_star=None,
                  anchor=None, hreflang=None, type_hint=None):
-        """
-        Add a link header to the response.
+        """Add a link header to the response.
 
-        See also: https://tools.ietf.org/html/rfc5988
+        (See also: RFC 5988, Section 1)
 
         Note:
             Calling this method repeatedly will cause each link to be
@@ -375,10 +531,11 @@ class Response(object):
                 link. Will be converted to a URI, if necessary, per
                 RFC 3987, Section 3.1.
             rel (str): Relation type of the link, such as "next" or
-                "bookmark". See also http://goo.gl/618GHr for a list
-                of registered link relation types.
+                "bookmark".
 
-        Kwargs:
+                (See also: http://www.iana.org/assignments/link-relations/link-relations.xhtml)
+
+        Keyword Args:
             title (str): Human-readable label for the destination of
                 the link (default ``None``). If the title includes non-ASCII
                 characters, you will need to use `title_star` instead, or
@@ -462,6 +619,12 @@ class Response(object):
         if anchor is not None:
             value += '; anchor="' + uri_encode(anchor) + '"'
 
+        # NOTE(kgriffs): uwsgi fails with a TypeError if any header
+        # is not a str, so do the conversion here. It's actually
+        # faster to not do an isinstance check. str() will encode
+        # to US-ASCII.
+        value = str(value)
+
         _headers = self._headers
         if 'link' in _headers:
             _headers['link'] += ', ' + value
@@ -470,18 +633,23 @@ class Response(object):
 
     cache_control = header_property(
         'Cache-Control',
-        """Sets the Cache-Control header.
+        """Set the Cache-Control header.
 
         Used to set a list of cache directives to use as the value of the
         Cache-Control header. The list will be joined with ", " to produce
         the value for the header.
 
         """,
-        lambda v: ', '.join(v))
+        format_header_value_list)
 
     content_location = header_property(
         'Content-Location',
-        'Sets the Content-Location header.',
+        """Set the Content-Location header.
+
+        This value will be URI encoded per RFC 3986. If the value that is
+        being set is already URI encoded it should be decoded first or the
+        header should be set manually using the set_header method.
+        """,
         uri_encode)
 
     content_range = header_property(
@@ -500,21 +668,40 @@ class Response(object):
             case, raising ``falcon.HTTPRangeNotSatisfiable`` will do the right
             thing.
 
-            See also: http://goo.gl/Iglhp
+        (See also: RFC 7233, Section 4.2)
         """,
         format_range)
 
     content_type = header_property(
         'Content-Type',
-        'Sets the Content-Type header.')
+        """Sets the Content-Type header.
+
+        The ``falcon`` module provides a number of constants for
+        common media types, including ``falcon.MEDIA_JSON``,
+        ``falcon.MEDIA_MSGPACK``, ``falcon.MEDIA_YAML``,
+        ``falcon.MEDIA_XML``, ``falcon.MEDIA_HTML``,
+        ``falcon.MEDIA_JS``, ``falcon.MEDIA_TEXT``,
+        ``falcon.MEDIA_JPEG``, ``falcon.MEDIA_PNG``,
+        and ``falcon.MEDIA_GIF``.
+        """)
+
+    downloadable_as = header_property(
+        'Content-Disposition',
+        """Set the Content-Disposition header using the given filename.
+
+        The value will be used for the *filename* directive. For example,
+        given ``'report.pdf'``, the Content-Disposition header would be set
+        to: ``'attachment; filename="report.pdf"'``.
+        """,
+        format_content_disposition)
 
     etag = header_property(
         'ETag',
-        'Sets the ETag header.')
+        'Set the ETag header.')
 
     last_modified = header_property(
         'Last-Modified',
-        """Sets the Last-Modified header. Set to a ``datetime`` (UTC) instance.
+        """Set the Last-Modified header. Set to a ``datetime`` (UTC) instance.
 
         Note:
             Falcon will format the ``datetime`` as an HTTP date string.
@@ -523,12 +710,17 @@ class Response(object):
 
     location = header_property(
         'Location',
-        'Sets the Location header.',
+        """Set the Location header.
+
+        This value will be URI encoded per RFC 3986. If the value that is
+        being set is already URI encoded it should be decoded first or the
+        header should be set manually using the set_header method.
+        """,
         uri_encode)
 
     retry_after = header_property(
         'Retry-After',
-        """Sets the Retry-After header.
+        """Set the Retry-After header.
 
         The expected value is an integral number of seconds to use as the
         value for the header. The HTTP-date syntax is not supported.
@@ -540,29 +732,54 @@ class Response(object):
         """Value to use for the Vary header.
 
         Set this property to an iterable of header names. For a single
-        asterisk or field value, simply pass a single-element ``list`` or
-        ``tuple``.
+        asterisk or field value, simply pass a single-element ``list``
+        or ``tuple``.
 
-        "Tells downstream proxies how to match future request headers
-        to decide whether the cached response can be used rather than
-        requesting a fresh one from the origin server."
-
-        (Wikipedia)
-
-        See also: http://goo.gl/NGHdL
+        The "Vary" header field in a response describes what parts of
+        a request message, aside from the method, Host header field,
+        and request target, might influence the origin server's
+        process for selecting and representing this response.  The
+        value consists of either a single asterisk ("*") or a list of
+        header field names (case-insensitive).
 
+        (See also: RFC 7231, Section 7.1.4)
         """,
-        lambda v: ', '.join(v))
+        format_header_value_list)
 
-    def _encode_header(self, name, value, py2=PY2):
-        if py2:
-            if isinstance(name, unicode):
-                name = name.encode('ISO-8859-1')
+    accept_ranges = header_property(
+        'Accept-Ranges',
+        """Set the Accept-Ranges header.
 
-            if isinstance(value, unicode):
-                value = value.encode('ISO-8859-1')
+        The Accept-Ranges header field indicates to the client which
+        range units are supported (e.g. "bytes") for the target
+        resource.
 
-        return name, value
+        If range requests are not supported for the target resource,
+        the header may be set to "none" to advise the client not to
+        attempt any such requests.
+
+        Note:
+            "none" is the literal string, not Python's built-in ``None``
+            type.
+
+        """)
+
+    def _set_media_type(self, media_type=None):
+        """Wrapper around set_header to set a content-type.
+
+        Args:
+            media_type: Media type to use for the Content-Type
+                header.
+
+        """
+
+        # PERF(kgriffs): Using "in" like this is faster than using
+        # dict.setdefault (tested on py27).
+        set_content_type = (media_type is not None and
+                            'content-type' not in self._headers)
+
+        if set_content_type:
+            self.set_header('content-type', media_type)
 
     def _wsgi_headers(self, media_type=None, py2=PY2):
         """Convert headers into the format expected by WSGI servers.
@@ -574,14 +791,7 @@ class Response(object):
         """
 
         headers = self._headers
-
-        # PERF(kgriffs): Using "in" like this is faster than using
-        # dict.setdefault (tested on py27).
-        set_content_type = (media_type is not None and
-                            'content-type' not in headers)
-
-        if set_content_type:
-            headers['content-type'] = media_type
+        self._set_media_type(media_type)
 
         if py2:
             # PERF(kgriffs): Don't create an extra list object if
@@ -602,3 +812,47 @@ class Response(object):
             items += [('set-cookie', c.OutputString())
                       for c in self._cookies.values()]
         return items
+
+
+class ResponseOptions(object):
+    """Defines a set of configurable response options.
+
+    An instance of this class is exposed via :any:`API.resp_options` for
+    configuring certain :py:class:`~.Response` behaviors.
+
+    Attributes:
+        secure_cookies_by_default (bool): Set to ``False`` in development
+            environments to make the `secure` attribute for all cookies
+            default to ``False``. This can make testing easier by
+            not requiring HTTPS. Note, however, that this setting can
+            be overridden via `set_cookie()`'s `secure` kwarg.
+
+        default_media_type (str): The default Internet media type (RFC 2046) to
+            use when deserializing a response. This value is normally set to the
+            media type provided when a :class:`falcon.API` is initialized;
+            however, if created independently, this will default to the
+            ``DEFAULT_MEDIA_TYPE`` specified by Falcon.
+
+        media_handlers (Handlers): A dict-like object that allows you to
+            configure the media-types that you would like to handle.
+            By default, a handler is provided for the ``application/json``
+            media type.
+
+        static_media_types (dict): A mapping of dot-prefixed file extensions to
+            Internet media types (RFC 2046). Defaults to ``mimetypes.types_map``
+            after calling ``mimetypes.init()``.
+    """
+    __slots__ = (
+        'secure_cookies_by_default',
+        'default_media_type',
+        'media_handlers',
+        'static_media_types',
+    )
+
+    def __init__(self):
+        self.secure_cookies_by_default = True
+        self.default_media_type = DEFAULT_MEDIA_TYPE
+        self.media_handlers = Handlers()
+
+        mimetypes.init()
+        self.static_media_types = mimetypes.types_map
