@@ -12,9 +12,20 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from falcon.routing.compiled import CompiledRouter
+"""Default router and utility functions.
+
+This package implements Falcon's default routing engine, field converter
+classes, and utility functions to aid in the implementation of custom
+routers.
+"""
+
+from falcon.routing.compiled import CompiledRouter, CompiledRouterOptions  # NOQA
+from falcon.routing.static import StaticRoute  # NOQA
 from falcon.routing.util import create_http_method_map  # NOQA
+from falcon.routing.util import map_http_methods  # NOQA
+from falcon.routing.util import set_default_responders  # NOQA
 from falcon.routing.util import compile_uri_template  # NOQA
+from falcon.routing.converters import *  # NOQA
 
 
 DefaultRouter = CompiledRouter
