@@ -12,14 +12,27 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+"""Testing utilities.
+
+This module contains various testing utilities that can be accessed
+directly from the `testing` package::
+
+    from falcon import testing
+
+    wsgi_environ = testing.create_environ()
+
+"""
+
 import cgi
-import random
+import contextlib
 import io
+import random
 import sys
 
 import six
 
-from falcon.util import uri, http_now
+from falcon.util import http_now, uri
+
 
 # Constants
 DEFAULT_HOST = 'falconframework.org'
@@ -108,6 +121,9 @@ def create_environ(path='/', query_string='', protocol='HTTP/1.1',
 
     """
 
+    if query_string and query_string.startswith('?'):
+        raise ValueError("query_string should not start with '?'")
+
     body = io.BytesIO(body.encode('utf-8')
                       if isinstance(body, six.text_type) else body)
 
@@ -115,6 +131,23 @@ def create_environ(path='/', query_string='', protocol='HTTP/1.1',
     # the paths before setting PATH_INFO
     path = uri.decode(path)
 
+    if six.PY3:
+        # NOTE(kgriffs): The decoded path may contain UTF-8 characters.
+        # But according to the WSGI spec, no strings can contain chars
+        # outside ISO-8859-1. Therefore, to reconcile the URI
+        # encoding standard that allows UTF-8 with the WSGI spec
+        # that does not, WSGI servers tunnel the string via
+        # ISO-8859-1. falcon.testing.create_environ() mimics this
+        # behavior, e.g.:
+        #
+        #   tunnelled_path = path.encode('utf-8').decode('iso-8859-1')
+        #
+        # falcon.Request does the following to reverse the process:
+        #
+        #   path = tunnelled_path.encode('iso-8859-1').decode('utf-8', 'replace')
+        #
+        path = path.encode('utf-8').decode('iso-8859-1')
+
     if six.PY2 and isinstance(path, six.text_type):
         path = path.encode('utf-8')
 
@@ -124,6 +157,12 @@ def create_environ(path='/', query_string='', protocol='HTTP/1.1',
     else:
         port = str(port)
 
+    # NOTE(kgriffs): Judging by the algorithm given in PEP-3333 for
+    # reconstructing the URL, SCRIPT_NAME is expected to contain a
+    # preceding slash character.
+    if app and not app.startswith('/'):
+        app = '/' + app
+
     env = {
         'SERVER_PROTOCOL': protocol,
         'SERVER_SOFTWARE': 'gunicorn/0.17.0',
@@ -174,6 +213,29 @@ def create_environ(path='/', query_string='', protocol='HTTP/1.1',
     return env
 
 
+@contextlib.contextmanager
+def redirected(stdout=sys.stdout, stderr=sys.stderr):
+    """
+    A context manager to temporarily redirect stdout or stderr
+
+    e.g.:
+
+    with redirected(stderr=os.devnull):
+        ...
+    """
+
+    old_stdout, old_stderr = sys.stdout, sys.stderr
+    sys.stdout, sys.stderr = stdout, stderr
+    try:
+        yield
+    finally:
+        sys.stderr, sys.stdout = old_stderr, old_stdout
+
+# ---------------------------------------------------------------------
+# Private
+# ---------------------------------------------------------------------
+
+
 def _add_headers_to_environ(env, headers):
     if not isinstance(headers, dict):
         # Try to convert
