@@ -12,6 +12,18 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+"""URI utilities.
+
+This module provides utility functions to parse, encode, decode, and
+otherwise manipulate a URI. These functions are not available directly
+in the `falcon` module, and so must be explicitly imported::
+
+    from falcon import uri
+
+    name, port = uri.parse_host('example.org:8080')
+
+"""
+
 import six
 
 # NOTE(kgriffs): See also RFC 3986
@@ -48,6 +60,7 @@ def _create_char_encoder(allowed_chars):
 def _create_str_encoder(is_value):
 
     allowed_chars = _UNRESERVED if is_value else _ALL_ALLOWED
+    allowed_chars_plus_percent = allowed_chars + '%'
     encode_char = _create_char_encoder(allowed_chars)
 
     def encoder(uri):
@@ -55,10 +68,32 @@ def _create_str_encoder(is_value):
         if not uri.rstrip(allowed_chars):
             return uri
 
+        if not uri.rstrip(allowed_chars_plus_percent):
+            # NOTE(kgriffs): There's a good chance the string has already
+            # been escaped. Do one more check to increase our certainty.
+            tokens = uri.split('%')
+            for token in tokens[1:]:
+                hex_octet = token[:2]
+
+                if not len(hex_octet) == 2:
+                    break
+
+                if not (hex_octet[0] in _HEX_DIGITS and
+                        hex_octet[1] in _HEX_DIGITS):
+                    break
+            else:
+                # NOTE(kgriffs): All percent-encoded sequences were
+                # valid, so assume that the string has already been
+                # encoded.
+                return uri
+
+            # NOTE(kgriffs): At this point we know there is at least
+            # one unallowed percent character. We are going to assume
+            # that everything should be encoded. If the string is
+            # partially encoded, the caller will need to normalize it
+            # before passing it in here.
+
         # Convert to a byte array if it is not one already
-        #
-        # NOTE(kgriffs): Code coverage disabled since in Py3K the uri
-        # is always a text type, so we get a failure for that tox env.
         if isinstance(uri, six.text_type):
             uri = uri.encode('utf-8')
 
@@ -93,7 +128,7 @@ Args:
 
 Returns:
     str: An escaped version of `uri`, where all disallowed characters
-        have been percent-encoded.
+    have been percent-encoded.
 
 """
 
@@ -125,11 +160,11 @@ Args:
 
 Returns:
     str: An escaped version of `uri`, where all disallowed characters
-        have been percent-encoded.
+    have been percent-encoded.
 
 """
 
-if six.PY2:
+if six.PY2:  # NOQA: C901 - Work around a bug in flake8 McCabe scoring
 
     # This map construction is based on urllib
     _HEX_TO_BYTE = dict((a + b, (chr(int(a + b, 16)), int(a + b, 16)))
@@ -139,7 +174,7 @@ if six.PY2:
     def decode(encoded_uri):
         """Decodes percent-encoded characters in a URI or query string.
 
-        This function models the behavior of `urllib.parse.unquote_plus`, but
+        This function models the behavior of `urllib.unquote_plus`, but
         is faster. It is also more robust, in that it will decode escaped
         UTF-8 mutibyte sequences.
 
@@ -148,8 +183,8 @@ if six.PY2:
 
         Returns:
             str: A decoded URL. Will be of type ``unicode`` on Python 2 IFF the
-                URL contained escaped non-ASCII characters, in which case
-                UTF-8 is assumed per RFC 3986.
+            URL contained escaped non-ASCII characters, in which case
+            UTF-8 is assumed per RFC 3986.
 
         """
 
@@ -178,10 +213,11 @@ if six.PY2:
         decoded_uri = tokens[0]
         for token in tokens[1:]:
             token_partial = token[:2]
-            if token_partial in _HEX_TO_BYTE:
+            try:
                 char, byte = _HEX_TO_BYTE[token_partial]
-            else:
+            except KeyError:
                 char, byte = '%', 0
+
             decoded_uri += char + (token[2:] if byte else token)
             only_ascii = only_ascii and (byte <= 127)
 
@@ -236,9 +272,9 @@ else:
         decoded_uri = tokens[0]
         for token in tokens[1:]:
             token_partial = token[:2]
-            if token_partial in _HEX_TO_BYTE:
+            try:
                 decoded_uri += _HEX_TO_BYTE[token_partial] + token[2:]
-            else:
+            except KeyError:
                 # malformed percentage like "x=%" or "y=%+"
                 decoded_uri += b'%' + token
 
@@ -246,11 +282,12 @@ else:
         return decoded_uri.decode('utf-8', 'replace')
 
 
-def parse_query_string(query_string, keep_blank_qs_values=False):
+def parse_query_string(query_string, keep_blank_qs_values=False,
+                       parse_qs_csv=True):
     """Parse a query string into a dict.
 
     Query string parameters are assumed to use standard form-encoding. Only
-    parameters with values are parsed. for example, given 'foo=bar&flag',
+    parameters with values are returned. For example, given 'foo=bar&flag',
     this function would ignore 'flag' unless the `keep_blank_qs_values` option
     is set.
 
@@ -269,13 +306,21 @@ def parse_query_string(query_string, keep_blank_qs_values=False):
 
     Args:
         query_string (str): The query string to parse.
-        keep_blank_qs_values (bool): If set to ``True``, preserves boolean
-            fields and fields with no content as blank strings.
+        keep_blank_qs_values (bool): Set to ``True`` to return fields even if
+            they do not have a value (default ``False``). For comma-separated
+            values, this option also determines whether or not empty elements
+            in the parsed list are retained.
+        parse_qs_csv: Set to ``False`` in order to disable splitting query
+            parameters on ``,`` (default ``True``). Depending on the user agent,
+            encoding lists as multiple occurrences of the same parameter might
+            be preferable. In this case, setting `parse_qs_csv` to ``False``
+            will cause the framework to treat commas as literal characters in
+            each occurring parameter value.
 
     Returns:
         dict: A dictionary of (*name*, *value*) pairs, one per query
-            parameter. Note that *value* may be a single ``str``, or a
-            ``list`` of ``str``.
+        parameter. Note that *value* may be a single ``str``, or a
+        ``list`` of ``str``.
 
     Raises:
         TypeError: `query_string` was not a ``str``.
@@ -284,6 +329,8 @@ def parse_query_string(query_string, keep_blank_qs_values=False):
 
     params = {}
 
+    is_encoded = '+' in query_string or '%' in query_string
+
     # PERF(kgriffs): This was found to be faster than using a regex, for
     # both short and long query strings. Tested on both CPython 2.7 and 3.4,
     # and on PyPy 2.3.
@@ -294,7 +341,8 @@ def parse_query_string(query_string, keep_blank_qs_values=False):
 
         # Note(steffgrez): Falcon first decode name parameter for handle
         # utf8 character.
-        k = decode(k)
+        if is_encoded:
+            k = decode(k)
 
         # NOTE(steffgrez): Falcon decode value at the last moment. So query
         # parser won't mix up between percent-encoded comma (as value) and
@@ -303,13 +351,17 @@ def parse_query_string(query_string, keep_blank_qs_values=False):
             # The key was present more than once in the POST data.  Convert to
             # a list, or append the next value to the list.
             old_value = params[k]
+
+            if is_encoded:
+                v = decode(v)
+
             if isinstance(old_value, list):
-                old_value.append(decode(v))
+                old_value.append(v)
             else:
-                params[k] = [old_value, decode(v)]
+                params[k] = [old_value, v]
 
         else:
-            if ',' in v:
+            if parse_qs_csv and ',' in v:
                 # NOTE(kgriffs): Falcon supports a more compact form of
                 # lists, in which the elements are comma-separated and
                 # assigned to a single param instance. If it turns out that
@@ -324,8 +376,10 @@ def parse_query_string(query_string, keep_blank_qs_values=False):
                     params[k] = [decode(element) for element in v if element]
                 else:
                     params[k] = [decode(element) for element in v]
-            else:
+            elif is_encoded:
                 params[k] = decode(v)
+            else:
+                params[k] = v
 
     return params
 
@@ -341,14 +395,16 @@ def parse_host(host, default_port=None):
     Args:
         host (str): Host string to parse, optionally containing a
             port number.
-        default_port (int, optional): Port number to return when
-            the host string does not contain one (default ``None``).
+
+    Keyword Arguments:
+        default_port (int): Port number to return when the host string
+            does not contain one (default ``None``).
 
     Returns:
         tuple: A parsed (*host*, *port*) tuple from the given
-            host string, with the port converted to an ``int``.
-            If the host string does not specify a port, `default_port` is
-            used instead.
+        host string, with the port converted to an ``int``.
+        If the host string does not specify a port, `default_port` is
+        used instead.
 
     """
 
