@@ -1,9 +1,8 @@
 
 import falcon
-import falcon.testing as testing
 
 
-class TestResponseBody(testing.TestBase):
+class TestResponseBody(object):
 
     def test_append_body(self):
         text = 'Hello beautiful world! '
@@ -14,4 +13,9 @@ class TestResponseBody(testing.TestBase):
             resp.body += token
             resp.body += ' '
 
-        self.assertEqual(resp.body, text)
+        assert resp.body == text
+
+    def test_response_repr(self):
+        resp = falcon.Response()
+        _repr = '<%s: %s>' % (resp.__class__.__name__, resp.status)
+        assert resp.__repr__() == _repr
