@@ -1,11 +1,23 @@
 import io
 
-import falcon.testing as testing
+import pytest
 import six
 
+import falcon
+import falcon.testing as testing
+
 unicode_message = u'Unicode: \x80'
 
 
+@pytest.fixture
+def client():
+    app = falcon.API()
+
+    tehlogger = LoggerResource()
+    app.add_route('/logger', tehlogger)
+    return testing.TestClient(app)
+
+
 class LoggerResource:
 
     def on_get(self, req, resp):
@@ -15,13 +27,9 @@ class LoggerResource:
         req.log_error(unicode_message.encode('utf-8'))
 
 
-class TestWSGIError(testing.TestBase):
-
-    def before(self):
-        self.tehlogger = LoggerResource()
-
-        self.api.add_route('/logger', self.tehlogger)
+class TestWSGIError(object):
 
+    def setup_method(self, method):
         self.wsgierrors_buffer = io.BytesIO()
 
         if six.PY3:
@@ -34,21 +42,20 @@ class TestWSGIError(testing.TestBase):
             # with undefined encoding, so do the encoding manually.
             self.wsgierrors = self.wsgierrors_buffer
 
-    def test_responder_logged_bytestring(self):
-        self.simulate_request('/logger', wsgierrors=self.wsgierrors,
-                              query_string='amount=10')
+    def test_responder_logged_bytestring(self, client):
+        client.simulate_request(path='/logger',
+                                wsgierrors=self.wsgierrors,
+                                query_string='amount=10')
 
         log = self.wsgierrors_buffer.getvalue()
 
-        self.assertIn(unicode_message.encode('utf-8'), log)
-        self.assertIn(b'?amount=10', log)
-
-    def test_responder_logged_unicode(self):
-        if six.PY3:
-            self.skipTest('Test only applies to Python 2')
-
-        self.simulate_request('/logger', wsgierrors=self.wsgierrors,
-                              method='HEAD')
+        assert unicode_message.encode('utf-8') in log
+        assert b'?amount=10' in log
 
+    @pytest.mark.skipif(six.PY3, reason='Test only applies to Python 2')
+    def test_responder_logged_unicode(self, client):
+        client.simulate_request(path='/logger',
+                                wsgierrors=self.wsgierrors,
+                                method='HEAD')
         log = self.wsgierrors_buffer.getvalue()
-        self.assertIn(unicode_message, log.decode('utf-8'))
+        assert unicode_message in log.decode('utf-8')
